/*****************************************************************************
**																			**
**	 Name: 	UART  Software Interface										**	
**																			**
******************************************************************************

(C) Copyright 2003 - Analog Devices, Inc.  All rights reserved.

File Name:		UART_LIbrary.c

Date Modified:	10/07/03	ported to C by TL	Rev 0.1

Software:       VisualDSP++ 3.5 alpha

Purpose:		This file contains some library functions
                commonly used on UART processing.				

******************************************************************************/

#include "system.h"

/*
int UART_detectAutobaud(void)

Assuming 8 data bits, this functions expects a '@' (ASCII 0x40) character 
on the UART RX pin. 

Timer 2 performs the autobaud detection. Also special support for
half-duplex systems is provided. 

Input parameters: none
Return values: timer period value
*/
section ("L1_code") int UART_detectAutobaud(void) {
	int period = 0;

#ifndef	STANDALONE
	printf("Press \"@\" key for UART autobaud detection \n");
#endif

	// First of all, enable UART clock. It is required for autobaud detection on 
	// silicon revision 0.0.
	*pUART_GCTL = UCEN;	
				
	// Activate Loopback mode in order the receive channel is disconnected 
	// from RX pin during autobaud detection.
#ifndef VDSP31
	*pUART_MCR = LOOP_ENA;			// for VisualDSP++ 3.5	
#else
	*pUART_MCR = LOOP;				// for VisualDSP++ 3.1	(project options -> compiler additional options -> -DVDSP31
#endif
	
	// Setup Timer 2 Controller to do the autobaud detection. Timer captures
	// duration between two falling edges. It expects a '@' (ASCII 0x40) 
	// character. 8-bit, no parity assumed.
	// Disable Timer 2 first, in case there was an unexpected history.  
	*pTIMER_DISABLE = TIMDIS2;
	*pTIMER_STATUS = TRUN2 | TOVL_ERR2 | TIMIL2;

	// Capture from UART RxD pin. Select period capture from falling edge to 
	// falling edge. Enable IRQ_ENA, but don't enable the interrupt at system 
	// level (SIC). 
	*pTIMER2_CONFIG = TIN_SEL | IRQ_ENA | PERIOD_CNT | WDTH_CAP;	
	
	// Start the timer and wait until the according interrupt latch bit TIMIL2
	// in the TIMER_STATUS register is set. Then, two falling edges on the RxD
	// pin have been detected.
	*pTIMER_ENABLE = TIMEN2;		
	while (!(*pTIMER_STATUS & TIMIL2)) { 
		// wait
	};

	// Disable Timer 2 again  
	*pTIMER_DISABLE = TIMDIS2;
	*pTIMER_STATUS = TRUN2 | TOVL_ERR2 | TIMIL2;
	
	// Save period value
	period = *pTIMER2_PERIOD;
 		
	// In order to support also half-duplex connections, we need to delay any 
	// transmission, in order the sent character does not overlap the autobaud
	// pattern.
	// Use Timer 2 to perform this delay. Note that the Period Register still
	// contains the proper value and the Width Register is not used.
	*pTIMER2_CONFIG = OUT_DIS | IRQ_ENA | PERIOD_CNT | PWM_OUT;	
	*pTIMER_ENABLE = TIMEN2;	
	while (!(*pTIMER_STATUS & TIMIL2)) { 
		// wait
	};

	// Disable Timer 2 again
	*pTIMER_DISABLE = TIMDIS2;
	*pTIMER_STATUS = TRUN2 | TOVL_ERR2 | TIMIL2;	
			
	// Deactive Loopback mode again
	*pUART_MCR = 0;	
	
	// done !	
	return period;
}

/*
void UART_init(int divisor)

Configures UART in 8 data bits, no parity, 1 stop bit mode.

Input parameters: divisor latch value
Return values:    none
*/
section ("L1_code") void UART_init(int divisor) {

	// First of all, enable UART clock. 
	*pUART_GCTL = UCEN;
	
	// Read period value and apply formula:  divisor = period/16/8
	// Write result to the two 8-bit DL registers (DLH:DLL).
	*pUART_LCR = DLAB;
	*pUART_DLL = divisor;
	*pUART_DLH = divisor>>8;

	// Clear DLAB again and set UART frame to 8 bits, no parity, 1 stop bit. 
	// This may differ in other scenarious.
	*pUART_LCR = WLS(8);			
}

/*
void UART_waitForTransferCompletion(void)

This function polls the TEMT bit in the LSR register and waits until
all data has been shifted out.

Input parameters: 	none
Return values: 		none
*/
section ("L1_code") void UART_waitForTransferCompletion(void) {	
	while (!(*pUART_LSR & TEMT)) { 
		// wait
	};
	ssync();
}
	
/*
void UART_disable(void)

This function disables the UART clock, but polls the TEMT bit before

Input parameters: 	none
Return values: 		none
*/
section ("L1_code") void UART_disable(void) {
	UART_waitForTransferCompletion();
	*pUART_GCTL = 0;
}

/*
void UART_putc(char c)

This function transmits a character by polling THRE bit in the LSR
register.

Input parameters:	the character to transmit
Return values:		none
*/
section ("L1_code") void UART_putc(char c) {
	while (!(*pUART_LSR & THRE)) { 
		// wait
	};
	*pUART_THR = c;		
}

/*
void UART_puts(char *c)

This function transmits a byte string by polling THRE bit in the LSR
register. The string must be NULL-terminated (C-style).

Input parameters: 	pointer to the string to transmit
Return values: 		none
*/
section ("L1_code") void UART_puts(char *c) {
	while (*c) {	
		UART_putc(*c);
		c++;
	}
}

/*
char UART_getc(void)

This function receives a character by polling the DR bit in the LSR
register.

Input parameters: 	none
Return values: 		character received from the RBR
*/
section ("L1_code") char UART_getc(void) {
	char c;
	
	while (!(*pUART_LSR & DR)) { 
	}

	// read Data
	c = *pUART_RBR;
	
	return c;
}

/*
void UART_gets(char *str)

This function receives an LF (LF = 'ENTER') terminated by polling the DR bit in the LSR
register.

Input parameters: 	pointer to the string
Return values: 		none
*/
section ("L1_code") void UART_gets(char *str) {
	int i;
	
	for(i = 0; i < 127; i++) {
		str[i] = UART_getc();
		//UART_putc(str[i]);
		if (str[i] == 13) {
			str[i] = 0;
			break;
		}
	}
	return;
}
